@@ -57,11 +57,11 @@ G_DEFINE_TYPE_WITH_CODE (
 		e_book_backend_google_source_authenticator_init))
 
 struct _EBookBackendGooglePrivate {
-	GList *bookviews;
-
 	EBookBackendCache *cache;
 	GMutex cache_lock;
 
+	/* For all the group-related members */
+	GRecMutex groups_lock;
 	/* Mapping from group ID to (human readable) group name */
 	GHashTable *groups_by_id;
 	/* Mapping from (human readable) group name to group ID */
@@ -71,7 +71,7 @@ struct _EBookBackendGooglePrivate {
 	/* Mapping entry ID to system_group_id */
 	GHashTable *system_groups_by_entry_id;
 	/* Time when the groups were last queried */
-	GTimeVal last_groups_update;
+	GTimeVal groups_last_update;
 
 	GDataAuthorizer *authorizer;
 	GDataService *service;
@@ -246,7 +246,13 @@ cache_add_contact (EBookBackend *backend,
 
 	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
 
+	g_rec_mutex_lock (&priv->groups_lock);
 	contact = e_contact_new_from_gdata_entry (entry, priv->groups_by_id, priv->system_groups_by_entry_id);
+	g_rec_mutex_unlock (&priv->groups_lock);
+
+	if (!contact)
+		return NULL;
+
 	e_contact_add_gdata_entry_xml (contact, entry);
 	g_mutex_lock (&priv->cache_lock);
 	e_book_backend_cache_add_contact (priv->cache, contact);
@@ -472,45 +478,6 @@ backend_is_authorized (EBookBackend *backend)
 	return gdata_service_is_authorized (priv->service);
 }
 
-static void
-on_contact_added (EBookBackend *backend,
-                  EContact *contact)
-{
-	EBookBackendGooglePrivate *priv;
-	GList *iter;
-
-	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-
-	for (iter = priv->bookviews; iter; iter = iter->next)
-		e_data_book_view_notify_update (E_DATA_BOOK_VIEW (iter->data), g_object_ref (contact));
-}
-
-static void
-on_contact_removed (EBookBackend *backend,
-                    const gchar *uid)
-{
-	EBookBackendGooglePrivate *priv;
-	GList *iter;
-
-	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-
-	for (iter = priv->bookviews; iter; iter = iter->next)
-		e_data_book_view_notify_remove (E_DATA_BOOK_VIEW (iter->data), g_strdup (uid));
-}
-
-static void
-on_contact_changed (EBookBackend *backend,
-                    EContact *contact)
-{
-	EBookBackendGooglePrivate *priv;
-	GList *iter;
-
-	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-
-	for (iter = priv->bookviews; iter; iter = iter->next)
-		e_data_book_view_notify_update (E_DATA_BOOK_VIEW (iter->data), g_object_ref (contact));
-}
-
 static GCancellable *
 start_operation (EBookBackend *backend,
                  guint32 opid,
@@ -518,7 +485,7 @@ start_operation (EBookBackend *backend,
                  const gchar *message)
 {
 	EBookBackendGooglePrivate *priv;
-	GList *iter;
+	GList *list, *link;
 
 	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
 
@@ -530,8 +497,14 @@ start_operation (EBookBackend *backend,
 	g_hash_table_insert (priv->cancellables, GUINT_TO_POINTER (opid), g_object_ref (cancellable));
 
 	/* Send out a status message to each view */
-	for (iter = priv->bookviews; iter; iter = iter->next)
-		e_data_book_view_notify_progress (E_DATA_BOOK_VIEW (iter->data), -1, message);
+	list = e_book_backend_list_views (backend);
+
+	for (link = list; link != NULL; link = g_list_next (link)) {
+		EDataBookView *view = E_DATA_BOOK_VIEW (link->data);
+		e_data_book_view_notify_progress (view, -1, message);
+	}
+
+	g_list_free_full (list, (GDestroyNotify) g_object_unref);
 
 	return cancellable;
 }
@@ -552,11 +525,16 @@ finish_operation (EBookBackend *backend,
 	}
 
 	if (g_hash_table_remove (priv->cancellables, GUINT_TO_POINTER (opid))) {
-		GList *iter;
+		GList *list, *link;
 
-		/* Send out a status message to each view */
-		for (iter = priv->bookviews; iter; iter = iter->next)
-			e_data_book_view_notify_complete (E_DATA_BOOK_VIEW (iter->data), book_error);
+		list = e_book_backend_list_views (backend);
+
+		for (link = list; link != NULL; link = g_list_next (link)) {
+			EDataBookView *view = E_DATA_BOOK_VIEW (link->data);
+			e_data_book_view_notify_complete (view, book_error);
+		}
+
+		g_list_free_full (list, (GDestroyNotify) g_object_unref);
 	}
 
 	g_clear_error (&book_error);
@@ -574,10 +552,13 @@ process_contact_finish (EBookBackend *backend,
 	was_cached = cache_has_contact (backend, gdata_entry_get_id (entry));
 	new_contact = cache_add_contact (backend, entry);
 
+	if (!new_contact)
+		return;
+
 	if (was_cached == TRUE) {
-		on_contact_changed (backend, new_contact);
+		e_book_backend_notify_update (backend, new_contact);
 	} else {
-		on_contact_added (backend, new_contact);
+		e_book_backend_notify_update (backend, new_contact);
 	}
 
 	g_object_unref (new_contact);
@@ -707,7 +688,7 @@ process_contact_cb (GDataEntry *entry,
 		/* Do we have this item in our cache? */
 		if (is_cached) {
 			cache_remove_contact (backend, uid);
-			on_contact_removed (backend, uid);
+			e_book_backend_notify_remove (backend, uid);
 		}
 	} else {
 		gchar *old_photo_etag = NULL;
@@ -887,6 +868,8 @@ process_group (GDataEntry *entry,
 	system_group_id = gdata_contacts_group_get_system_group_id (GDATA_CONTACTS_GROUP (entry));
 	is_deleted = gdata_contacts_group_is_deleted (GDATA_CONTACTS_GROUP (entry));
 
+	g_rec_mutex_lock (&priv->groups_lock);
+
 	if (system_group_id) {
 		g_debug ("Processing %ssystem group %s, %s", is_deleted ? "(deleted) " : "", system_group_id, uid);
 
@@ -928,6 +911,8 @@ process_group (GDataEntry *entry,
 		priv->groups_changed = cache_update_group (backend, uid, name) || priv->groups_changed;
 	}
 
+	g_rec_mutex_unlock (&priv->groups_lock);
+
 	g_free (name);
 }
 
@@ -954,17 +939,25 @@ get_groups_cb (GDataService *service,
 
 	if (!gdata_error) {
 		/* Update the update time */
-		g_get_current_time (&(priv->last_groups_update));
+		g_rec_mutex_lock (&priv->groups_lock);
+		g_get_current_time (&(priv->groups_last_update));
+		g_rec_mutex_unlock (&priv->groups_lock);
 	}
 
 	finish_operation (backend, -2, gdata_error);
 
+	g_rec_mutex_lock (&priv->groups_lock);
+
 	if (priv->groups_changed) {
 		priv->groups_changed = FALSE;
 
+		g_rec_mutex_unlock (&priv->groups_lock);
+
 		/* do the update for all contacts, like with an empty cache */
 		cache_set_last_update (backend, NULL);
 		get_new_contacts (backend);
+	} else {
+		g_rec_mutex_unlock (&priv->groups_lock);
 	}
 
 	g_object_unref (backend);
@@ -973,7 +966,21 @@ get_groups_cb (GDataService *service,
 }
 
 static void
-get_groups (EBookBackend *backend)
+get_groups_and_update_cache_cb (GDataService *service,
+				GAsyncResult *result,
+				EBookBackend *backend)
+{
+	g_object_ref (backend);
+
+	get_groups_cb (service, result, backend);
+	get_new_contacts (backend);
+
+	g_object_unref (backend);
+}
+
+static void
+get_groups (EBookBackend *backend,
+	    gboolean and_update_cache)
 {
 	EBookBackendGooglePrivate *priv;
 	GDataQuery *query;
@@ -984,17 +991,21 @@ get_groups (EBookBackend *backend)
 	g_debug (G_STRFUNC);
 	g_return_if_fail (backend_is_authorized (backend));
 
+	g_rec_mutex_lock (&priv->groups_lock);
+
 	/* Build our query */
 	query = GDATA_QUERY (gdata_contacts_query_new_with_limits (NULL, 0, G_MAXINT));
-	if (priv->last_groups_update.tv_sec != 0 || priv->last_groups_update.tv_usec != 0) {
-		gdata_query_set_updated_min (query, priv->last_groups_update.tv_sec);
+	if (priv->groups_last_update.tv_sec != 0 || priv->groups_last_update.tv_usec != 0) {
+		gdata_query_set_updated_min (query, priv->groups_last_update.tv_sec);
 		gdata_contacts_query_set_show_deleted (GDATA_CONTACTS_QUERY (query), TRUE);
 	}
 
-	g_object_ref (backend);
-
 	priv->groups_changed = FALSE;
 
+	g_rec_mutex_unlock (&priv->groups_lock);
+
+	g_object_ref (backend);
+
 	/* Run the query asynchronously */
 	cancellable = start_operation (backend, -2, NULL, _("Querying for updated groups…"));
 	gdata_contacts_service_query_groups_async (
@@ -1004,7 +1015,7 @@ get_groups (EBookBackend *backend)
 		(GDataQueryProgressCallback) process_group,
 		backend,
 		(GDestroyNotify) NULL,
-		(GAsyncReadyCallback) get_groups_cb,
+		(GAsyncReadyCallback) (and_update_cache ? get_groups_and_update_cache_cb : get_groups_cb),
 		backend);
 
 	g_object_unref (cancellable);
@@ -1056,11 +1067,15 @@ create_group (EBookBackend *backend,
 
 	system_group_id = e_contact_map_google_with_evo_group (category_name, FALSE);
 	if (system_group_id) {
-		const gchar *group_entry_id = g_hash_table_lookup (priv->system_groups_by_id, system_group_id);
+		gchar *group_entry_id;
+
+		g_rec_mutex_lock (&priv->groups_lock);
+		group_entry_id = g_strdup (g_hash_table_lookup (priv->system_groups_by_id, system_group_id));
+		g_rec_mutex_unlock (&priv->groups_lock);
 
 		g_return_val_if_fail (group_entry_id != NULL, NULL);
 
-		return g_strdup (group_entry_id);
+		return group_entry_id;
 	}
 
 	group = GDATA_ENTRY (gdata_contacts_group_new (NULL));
@@ -1081,8 +1096,12 @@ create_group (EBookBackend *backend,
 
 	/* Add the new group to the group mappings */
 	uid = g_strdup (gdata_entry_get_id (new_group));
+
+	g_rec_mutex_lock (&priv->groups_lock);
 	g_hash_table_replace (priv->groups_by_id, e_contact_sanitise_google_group_id (uid), g_strdup (category_name));
 	g_hash_table_replace (priv->groups_by_name, g_strdup (category_name), e_contact_sanitise_google_group_id (uid));
+	g_rec_mutex_unlock (&priv->groups_lock);
+
 	g_object_unref (new_group);
 
 	/* Update the cache. */
@@ -1109,8 +1128,9 @@ refresh_local_cache_cb (ESource *source,
 
 	g_debug ("Invoking cache refresh");
 
-	get_groups (backend);
-	get_new_contacts (backend);
+	/* The TRUE means the cache update will be run immediately
+	   after groups are updated */
+	get_groups (backend, TRUE);
 }
 
 static void
@@ -1140,8 +1160,14 @@ cache_refresh_if_needed (EBookBackend *backend)
 			refresh_local_cache_cb,
 			backend,
 			NULL);
-	} else if (g_hash_table_size (priv->system_groups_by_id) == 0) {
-		get_groups (backend);
+	} else {
+		g_rec_mutex_lock (&priv->groups_lock);
+		if (g_hash_table_size (priv->system_groups_by_id) == 0) {
+			g_rec_mutex_unlock (&priv->groups_lock);
+			get_groups (backend, FALSE);
+		} else {
+			g_rec_mutex_unlock (&priv->groups_lock);
+		}
 	}
 
 	return;
@@ -1434,9 +1460,6 @@ book_backend_google_dispose (GObject *object)
 	/* Cancel all outstanding operations */
 	google_cancel_all_operations (E_BOOK_BACKEND (object));
 
-	g_list_free_full (priv->bookviews, (GDestroyNotify) g_object_unref);
-	priv->bookviews = NULL;
-
 	if (priv->refresh_id > 0) {
 		e_source_refresh_remove_timeout (
 			e_backend_get_source (E_BACKEND (object)),
@@ -1470,6 +1493,7 @@ book_backend_google_finalize (GObject *object)
 	}
 
 	g_mutex_clear (&priv->cache_lock);
+	g_rec_mutex_clear (&priv->groups_lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_book_backend_google_parent_class)->finalize (object);
@@ -1735,6 +1759,8 @@ book_backend_google_create_contacts_sync (EBookBackend *backend,
 
 	g_warn_if_fail (backend_is_authorized (backend));
 
+	g_rec_mutex_lock (&priv->groups_lock);
+
 	/* Ensure the system groups have been fetched. */
 	if (g_hash_table_size (priv->system_groups_by_id) == 0)
 		get_groups_sync (backend, cancellable);
@@ -1748,6 +1774,8 @@ book_backend_google_create_contacts_sync (EBookBackend *backend,
 		_create_group, backend);
 	g_object_unref (contact);
 
+	g_rec_mutex_unlock (&priv->groups_lock);
+
 	/* Debug XML output */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
 	g_debug ("new entry with xml: %s", xml);
@@ -1810,8 +1838,10 @@ book_backend_google_create_contacts_sync (EBookBackend *backend,
 	}
 
 	contact = cache_add_contact (backend, GDATA_ENTRY (new_contact));
-	g_queue_push_tail (out_contacts, g_object_ref (contact));
-	g_object_unref (contact);
+	if (contact) {
+		g_queue_push_tail (out_contacts, g_object_ref (contact));
+		g_object_unref (contact);
+	}
 
 exit:
 	g_clear_object (&entry);
@@ -1902,6 +1932,8 @@ book_backend_google_modify_contacts_sync (EBookBackend *backend,
 		return FALSE;
 	}
 
+	g_rec_mutex_lock (&priv->groups_lock);
+
 	/* Ensure the system groups have been fetched. */
 	if (g_hash_table_size (priv->system_groups_by_id) == 0)
 		get_groups_sync (backend, cancellable);
@@ -1913,6 +1945,8 @@ book_backend_google_modify_contacts_sync (EBookBackend *backend,
 		priv->system_groups_by_id,
 		_create_group, backend);
 
+	g_rec_mutex_unlock (&priv->groups_lock);
+
 	/* Output debug XML */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
 	g_debug ("Before:\n%s", xml);
@@ -2013,14 +2047,17 @@ book_backend_google_modify_contacts_sync (EBookBackend *backend,
 		g_object_set_data_full (
 			G_OBJECT (new_contact), "photo", photo,
 			(GDestroyNotify) e_contact_photo_free);
+		photo = NULL;
 	} else {
 		g_object_set_data (
 			G_OBJECT (new_contact), "photo", NULL);
 	}
 
 	contact = cache_add_contact (backend, new_contact);
-	g_queue_push_tail (out_contacts, g_object_ref (contact));
-	g_object_unref (contact);
+	if (contact) {
+		g_queue_push_tail (out_contacts, g_object_ref (contact));
+		g_object_unref (contact);
+	}
 
 exit:
 	g_clear_object (&entry);
@@ -2177,20 +2214,14 @@ static void
 book_backend_google_start_view (EBookBackend *backend,
                                 EDataBookView *bookview)
 {
-	EBookBackendGooglePrivate *priv;
 	GQueue queue = G_QUEUE_INIT;
 	GError *error = NULL;
 
 	g_return_if_fail (E_IS_BOOK_BACKEND_GOOGLE (backend));
 	g_return_if_fail (E_IS_DATA_BOOK_VIEW (bookview));
 
-	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-
 	g_debug (G_STRFUNC);
 
-	priv->bookviews = g_list_append (priv->bookviews, bookview);
-
-	g_object_ref (bookview);
 	e_data_book_view_notify_progress (bookview, -1, _("Loading…"));
 
 	/* Ensure that we're ready to support a view */
@@ -2219,18 +2250,7 @@ static void
 book_backend_google_stop_view (EBookBackend *backend,
                                EDataBookView *bookview)
 {
-	EBookBackendGooglePrivate *priv;
-	GList *view;
-
-	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-
 	g_debug (G_STRFUNC);
-
-	/* Remove the view from the list of active views */
-	if ((view = g_list_find (priv->bookviews, bookview)) != NULL) {
-		priv->bookviews = g_list_delete_link (priv->bookviews, view);
-		g_object_unref (bookview);
-	}
 }
 
 static void
@@ -2344,6 +2364,7 @@ e_book_backend_google_init (EBookBackendGoogle *backend)
 	backend->priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
 
 	g_mutex_init (&backend->priv->cache_lock);
+	g_rec_mutex_init (&backend->priv->groups_lock);
 
 	g_signal_connect (
 		backend, "notify::online",
