@@ -371,7 +371,8 @@ imapx_store_mailbox_attributes_to_flags (CamelIMAPXMailbox *mailbox)
 	const gchar *attribute;
 
 	attribute = CAMEL_IMAPX_LIST_ATTR_NOSELECT;
-	if (camel_imapx_mailbox_has_attribute (mailbox, attribute))
+	if (camel_imapx_mailbox_has_attribute (mailbox, attribute) &&
+	    !camel_imapx_mailbox_is_inbox (camel_imapx_mailbox_get_name (mailbox)))
 		store_info_flags |= CAMEL_STORE_INFO_FOLDER_NOSELECT;
 
 	attribute = CAMEL_IMAPX_LIST_ATTR_NOINFERIORS;
@@ -3302,3 +3303,12 @@ camel_imapx_store_ref_job (CamelIMAPXStore *imapx_store,
 
 	return job;
 }
+
+/* for debugging purposes only */
+void
+camel_imapx_store_dump_queue_status (CamelIMAPXStore *imapx_store)
+{
+	g_return_if_fail (CAMEL_IS_IMAPX_STORE (imapx_store));
+
+	camel_imapx_conn_manager_dump_queue_status (imapx_store->priv->con_man);
+}
